/* aacomp.c: computes the amino acid composition of a given sequence */
/* It reads the protein sequence in one-letter code from stdin and prints */
/* the result to stdout. Acceptable input formats are raw (i.e. nothing but */
/* the sequence) and FASTA (one header line only) */

/* Copyright Markus Hoenicka 991217 hoenicka_markus@compuserve.com */
/* $Id: aacomp.c,v 1.7 2000/07/11 04:34:15 markus Exp $ */

/* This program is pretty straightforward ANSI-C code without any bells */
/* and whistles, so it should compile on almost any system. Use something */
/* like (this works for the GNU C compiler on Linux and CygWin32): */
/* gcc -s -o aacomp aacomp.c */

/*  This program is free software; you can redistribute it and/or */
/*  modify it under the terms of the GNU General Public License */
/*  as published by the Free Software Foundation; either version 2 */
/*  of the License, or (at your option) any later version. */

/*  This program is distributed in the hope that it will be useful, */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/*  GNU General Public License for more details. */

/*  You should have received a copy of the GNU General Public License */
/*  along with this program; if not, write to the Free Software */
/*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>

#define raw 0
#define fasta 1
#define TRUE 1
#define FALSE 0
#define BUFSIZE 512 /* buffer size for first line in FASTA format */

/* advance function declarations, not enough to justify a header file */
int computeaa();
void printreport(int printalpha, int printstat, int printaroma, int printacid, int printbase, int printnonpolar, int printunchpolar, int printother);

/* define a few things to simplify the sequence handling. The following
 three constructs have all the same order of amino acids */

enum aminoacids{ALA, ARG, ASN, ASP, ASX, CYS, GLN, GLU, GLX, GLY, HIS, ILE, LEU, LYS, MET, PHE, PRO, SER, TERM, THR, TRP, TYR, UNKX, VAL};

char aanames[24][4] = {"ALA", "ARG", "ASN", "ASP", "ASX", "CYS", "GLN", "GLU", "GLX", "GLY", "HIS", "ILE", "LEU", "LYS", "MET", "PHE", "PRO", "SER", "TER", "THR", "TRP", "TYR", "UNK", "VAL"};

float aamolweights[24] = {71, 156, 114, 115, 114.5, 103, 128, 129, 128.5, 57, 137, 113, 113, 128, 131, 147, 97, 87, 0, 101, 186, 163, 119, 99}; /* these values have one water (18) subtracted from the individual amino acid molecular weights to correct for the peptide bonds in the protein */

/* other globals */

float molweight = 0; /* total molecular weight of the protein */
float acidmolweight = 0; /* molecular weight of all acidic amino acids */
float basemolweight = 0; /* molecular weight of all basic amino acids */
float unchpolarmolweight = 0; /* molecular weight of all uncharged polar amino acids */
float nonpolarmolweight = 0; /* molecular weight of all nonpolar amino acids */
float othermolweight = 0; /* molecular weight of all other amino acids */
float aromamolweight = 0; /* molecular weight of aromatic amino acids */

int aacount[24]; /* counts for the individual amino acids */

int count = 0; /* counts total number of valid amino acids */
int acidcount = 0; /* counts number of amino acids with acidic side chains */
int basecount = 0; /* counts number of amino acids with acidic side chains */
int unchpolarcount = 0; /* counts number of amino acids with acidic side chains */
int nonpolarcount = 0; /* counts number of amino acids with acidic side chains */
int othercount = 0; /* counts number of unknowns, ASX, GLX */
int aromacount = 0; /* counts number of aromatic amino acids TYR PHE TRP */
int informat = raw; /* the input file format */


/****************************************************************************/
/* the one and only main()                                                  */
/* aacomp returns zero if all went ok and nonzero if an error occurred      */
/****************************************************************************/
int main(int argc, char *argv[])
{
  int opt;
  int bailout = -1; /* -1 keep running; 0 stop without error; 1 stop with error */
  int printalpha = FALSE; /* this and next lines: output switches, default is off */
  int printstat = FALSE;
  int printaroma = FALSE;
  int printacid = FALSE;
  int printbase = FALSE;
  int printnonpolar = FALSE;
  int printunchpolar = FALSE;
  int printother = FALSE;

  /* get command line options */
  while((opt = getopt(argc, argv, "abchnoprst:uv")) != EOF)
  {
      switch(opt)
      {
      case 'a':
	printalpha = TRUE;
	break;
      case 'b':
	printacid = TRUE;
	break;
      case 'c':
	printbase = TRUE;
	break;
      case 'h':
	printf("Usage: aacomp [-a][-b][-c][-h][-n][-o][-p][-r][-s][-t type][-u][-v]\nReads sequence from stdin and prints the amino acid composition to stdout\nOptions: -a list all amino acids alphabetically\n         -b list basic amino acids\n         -c list acidic amino acids\n         -h prints this help\n         -n list nonpolar amino acids\n         -p list all amino acids,sort by side chain properties (same as -bcnou)\n         -o list other amino acids\n         -r list aromatic amino acids\n         -s print some statistics\n         -t select the input file type. May be raw (=default) or fasta\n         -u list uncharged polar amino acids\n         -v print version and program info\n");
	bailout = FALSE;
	break;
      case 'n':
	printnonpolar = TRUE;
	break;
      case 'o':
	printother = TRUE;
	break;
      case 'p':
	printacid = TRUE;
	printbase = TRUE;
	printnonpolar = TRUE;
	printunchpolar = TRUE;
	printother = TRUE;
	break;
      case 'r':
	printaroma = TRUE;
	break;
      case 's':
	printstat = TRUE;
	break;
      case 't':
	if (strcmp(optarg, "fasta") == 0)
	  informat = fasta;
	else if (strcmp(optarg, "raw") == 0)
	  informat = raw;
	else
	  {
	    fprintf(stderr, "-t: Specify raw or fasta input file type\n");
	    bailout = TRUE;
	  }
	break;
      case 'u':
	printunchpolar = TRUE;
	break;
      case 'v':
	printf("aacomp $Revision: 1.7 $. (C) Markus Hoenicka 1999 <hoenicka_markus@compuserve.com>\naacomp comes with ABSOLUTELY NO WARRANTY.  This is free software, and you are welcome to redistribute it under certain conditions; see the GNU Public License (GPL) at www.gnu.org\n");
	bailout = FALSE;
	break;
      case ':':
	fprintf(stderr, "Option argument missing. Use -h to invoke help.\n");
	bailout = TRUE;
	break;
      case '?':
	fprintf(stderr, "Unknown option: %c. Use -h to invoke help.\n", optopt);
	bailout = TRUE;
	break;
      }
  }
  
  /* gracefully exit if some error was detected in the arguments or if help was requested */
  if (bailout > -1)
    exit (bailout);

  /* Don't let the user clueless if he didn't select one of the output switches */
  if (printalpha+printstat+printaroma+printacid+printbase+printnonpolar+printunchpolar+printother == FALSE) {
    fprintf(stderr, "No output requested. Use one or more of -a, -b, -c, -n, -p, -s, -u, or use -h to invoke help.\n");
    exit (1);
  }

  /* compute the data */
  if (computeaa() == 0) {
    fprintf(stderr, "No sequence found. Stop.\n");
    exit (1);
  }

  printreport(printalpha, printstat, printaroma, printacid, printbase, printnonpolar, printunchpolar, printother);

  exit (0);
}

/****************************************************************************/
/* computeaa(): calculates the amounts of amino acids                       */
/* Return: int: number of amino acids in the sequence, thus zero indicates  */
/*              an error                                                    */
/****************************************************************************/
int computeaa()
{
  int c; /* one character from the amino acid sequence */
  int i; /* as always, a loop variable */
  enum aminoacids aa; /* specifies the current amino acid */
  char inbuf[BUFSIZE]; /* buffer for the first line in a FASTA file */
  char *readresult; /* pointer to fgets buffer */

  /* some initialization */
  for (i = 0; i < 24; i++)
    {
      aacount[i] = 0;
    }

  /* if we read a FASTA file, strip the first line */
  if (informat == fasta) {
    do {
      readresult = fgets(inbuf, BUFSIZE-1, stdin);
    }
    while (inbuf[strlen(inbuf) - 1] != '\n' && readresult != NULL);
  }

  /* now read the rest of the file, increment counters as needed, add up the MWs */
  do
    {
      c = getchar();
      switch(c)
	{
	case 'G':
	  aa = GLY;
	  molweight += aamolweights[aa];
	  nonpolarmolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  nonpolarcount += 1;
	  break;
	case 'A':
	  aa = ALA;
	  molweight += aamolweights[aa];
	  nonpolarmolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  nonpolarcount += 1;
	  break;
	case 'V':
	  aa = VAL;
	  molweight += aamolweights[aa];
	  nonpolarmolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  nonpolarcount += 1;
	  break;
	case 'L':
	  aa = LEU;
	  molweight += aamolweights[aa];
	  nonpolarmolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  nonpolarcount += 1;
	  break;
	case 'I':
	  aa = ILE;
	  molweight += aamolweights[aa];
	  nonpolarmolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  nonpolarcount += 1;
	  break;
	case 'M':
	  aa = MET;
	  molweight += aamolweights[aa];
	  nonpolarmolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  nonpolarcount += 1;
	  break;
	case 'F':
	  aa = PHE;
	  molweight += aamolweights[aa];
	  nonpolarmolweight += aamolweights[aa];
	  aromamolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  nonpolarcount += 1;
	  aromacount += 1;
	  break;
	case 'P':
	  aa = PRO;
	  molweight += aamolweights[aa];
	  nonpolarmolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  nonpolarcount += 1;
	  break;
	case 'S':
	  aa = SER;
	  molweight += aamolweights[aa];
	  unchpolarmolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  unchpolarcount += 1;
	  break;
	case 'T':
	  aa = THR;
	  molweight += aamolweights[aa];
	  unchpolarmolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  unchpolarcount += 1;
	  break;
	case 'C':
	  aa = CYS;
	  molweight += aamolweights[aa];
	  nonpolarmolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  nonpolarcount += 1;
	  break;
	case 'N':
	  aa = ASN;
	  molweight += aamolweights[aa];
	  unchpolarmolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  unchpolarcount += 1;
	  break;
	case 'Q':
	  aa = GLN;
	  molweight += aamolweights[aa];
	  unchpolarmolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  unchpolarcount += 1;
	  break;
	case 'Y':
	  aa = TYR;
	  molweight += aamolweights[aa];
	  unchpolarmolweight += aamolweights[aa];
	  aromamolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  unchpolarcount += 1;
	  aromacount += 1;
	  break;
	case 'W':
	  aa = TRP;
	  molweight += aamolweights[aa];
	  nonpolarmolweight += aamolweights[aa];
	  aromamolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  nonpolarcount += 1;
	  aromacount += 1;
	  break;
	case 'D':
	  aa = ASP;
	  molweight += aamolweights[aa];
	  acidmolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  acidcount += 1;
	  break;
	case 'E':
	  aa = GLU;
	  molweight += aamolweights[aa];
	  acidmolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  acidcount += 1;
	  break;
	case 'H':
	  aa = HIS;
	  molweight += aamolweights[aa];
	  basemolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  basecount += 1;
	  break;
	case 'K':
	  aa = LYS;
	  molweight += aamolweights[aa];
	  basemolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  basecount += 1;
	  break;
	case 'R':
	  aa = ARG;
	  molweight += aamolweights[aa];
	  basemolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  basecount += 1;
	  break;
	case 'X':
	  aa = UNKX;
	  molweight += aamolweights[aa];
	  othermolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  othercount += 1;
	  break;
	case 'B':
	  aa = ASX;
	  molweight += aamolweights[aa];
	  othermolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  othercount += 1;
	  break;
	case 'Z':
	  aa = GLX;
	  molweight += aamolweights[aa];
	  othermolweight += aamolweights[aa];
	  aacount[aa] += 1;
	  count += 1;
	  othercount += 1;
	  break;
	case '*':
	  aa = TERM;
	  aacount[aa] += 1;
	  break;
	}
    }
  while (c != EOF);

  /* correct molecular weights for one water. This may not be absolutely correct for */
  /* the partial MWs, as it compares a hypothetical protein consisting only of e.g. */
  /* acidic amino acids with the full protein */
  molweight += 18;

  /* return the number of amino acids found */
  return count;
}

/****************************************************************************/
/* printreport(): prints the results to stdout                              */
/* Arguments: sortside: if TRUE, print a list based on the side chains      */
/*            printalpha: if TRUE, print an alphabetic list                 */
/*            printstat: if TRUE, print some statistics                     */
/*            printaroma: if TRUE, print aromatic amino acids               */
/*            printacid = if TRUE, print acidic amino acids                 */
/*            printbase = if TRUE, print basic amino acids                  */
/*            printnonpolar = if TRUE, print nonpolar amino acids           */
/*            printunchpolar = if TRUE, print uncharged polar amino acids   */
/*            printother = if TRUE, print other amino acids                 */
/****************************************************************************/
void printreport(int printalpha, int printstat, int printaroma, int printacid, int printbase, int printnonpolar, int printunchpolar, int printother)
{
  int i, j; /* as always, loop variables */
  int first = TRUE; /* used in statistics output. If true, text is printed */
  float relcontent, mwcontent; /* will hold calculated values for relative */
                               /* content based on abundance and on molecular
				  weight, respectively */

  if (printstat == TRUE)
  {
    printf("# Molecular weight: %f\n# Total number of amino acids: %d\n", molweight, count);
    for (j = 0; j < count; j++) { /* loop over possible number of aa */
      for (i = 0; i < 24; i++) { /* loop over amino acids */
	if (aacount[i] == j) {
	  if (first == TRUE) {
	    printf("# Amino acids occurring %d times:", j);
	    first = FALSE;
	  }
	  printf(" %s", aanames[i]);
	}
      }
      if (first == FALSE) {
	printf("\n");
	first = TRUE;
      }
    }
  }

  if (printalpha == TRUE)
  {
  /* print header */
    printf("\n# AA\tnumber\trelcontent\tMW-content\n");
    for (i = 0; i < 24; i++)
      {
	relcontent = (float)aacount[i]/count;
	mwcontent = ((aacount[i]*aamolweights[i]) + (18*relcontent))/molweight;
	printf("%s\t%d\t%f\t%f\n", aanames[i], aacount[i], relcontent, mwcontent);
      }
  }

  if (printacid == TRUE)
  {
    /* Acidic side chains */
    printf("\n# Acidic side chains\n# AA\tnumber\trelcontent\tMW-content\n");

    if (printstat == TRUE)
    {
      relcontent = (float)acidcount/count;
      mwcontent = (acidmolweight + (18*relcontent))/molweight;
      printf("all\t%d\t%f\t%f\n", acidcount, relcontent, mwcontent);
    }

    relcontent = (float)aacount[ASP]/count;
    mwcontent = ((aacount[ASP]*aamolweights[ASP]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[ASP], aacount[ASP], relcontent, mwcontent);

    relcontent = (float)aacount[GLU]/count;
    mwcontent = ((aacount[GLU]*aamolweights[GLU]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[GLU], aacount[GLU], relcontent, mwcontent);
  }

  if (printbase == TRUE) {
    /* Basic side chains */
    printf("\n# Basic side chains\n# AA\tnumber\trelcontent\tMW-content\n");

    if (printstat == TRUE)
    {
      relcontent = (float)basecount/count;
      mwcontent = (basemolweight + (18*relcontent))/molweight;
      printf("all\t%d\t%f\t%f\n", basecount, relcontent, mwcontent);
    }

    relcontent = (float)aacount[ARG]/count;
    mwcontent = ((aacount[ARG]*aamolweights[ARG]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[ARG], aacount[ARG], relcontent, mwcontent);

    relcontent = (float)aacount[HIS]/count;
    mwcontent = ((aacount[HIS]*aamolweights[HIS]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[HIS], aacount[HIS], relcontent, mwcontent);

    relcontent = (float)aacount[LYS]/count;
    mwcontent = ((aacount[LYS]*aamolweights[LYS]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[LYS], aacount[LYS], relcontent, mwcontent);
  }

  if (printunchpolar == TRUE) {
    /* Uncharged polar side chains*/
    printf("\n# Uncharged polar side chains\n# AA\tnumber\trelcontent\tMW-content\n");

    if (printstat == TRUE)
    {
      relcontent = (float)unchpolarcount/count;
      mwcontent = (unchpolarmolweight + (18*relcontent))/molweight;
      printf("all\t%d\t%f\t%f\n", unchpolarcount, relcontent, mwcontent);
    }

    relcontent = (float)aacount[ASN]/count;
    mwcontent = ((aacount[ASN]*aamolweights[ASN]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[ASN], aacount[ASN], relcontent, mwcontent);

    relcontent = (float)aacount[GLN]/count;
    mwcontent = ((aacount[GLN]*aamolweights[GLN]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[GLN], aacount[GLN], relcontent, mwcontent);

    relcontent = (float)aacount[SER]/count;
    mwcontent = ((aacount[SER]*aamolweights[SER]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[SER], aacount[SER], relcontent, mwcontent);

    relcontent = (float)aacount[THR]/count;
    mwcontent = ((aacount[THR]*aamolweights[THR]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[THR], aacount[THR], relcontent, mwcontent);

    relcontent = (float)aacount[TYR]/count;
    mwcontent = ((aacount[TYR]*aamolweights[TYR]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[TYR], aacount[TYR], relcontent, mwcontent);
  }

  if (printnonpolar == TRUE) {
    /* Nonpolar side chains*/
    printf("\n# Nonpolar side chains\n# AA\tnumber\trelcontent\tMW-content\n");

    if (printstat == TRUE)
    {
      relcontent = (float)nonpolarcount/count;
      mwcontent = (nonpolarmolweight + (18*relcontent))/molweight;
      printf("all\t%d\t%f\t%f\n", nonpolarcount, relcontent, mwcontent);
    }

    relcontent = (float)aacount[ALA]/count;
    mwcontent = ((aacount[ALA]*aamolweights[ALA]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[ALA], aacount[ALA], relcontent, mwcontent);

    relcontent = (float)aacount[CYS]/count;
    mwcontent = ((aacount[CYS]*aamolweights[CYS]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[CYS], aacount[CYS], relcontent, mwcontent);

    relcontent = (float)aacount[GLY]/count;
    mwcontent = ((aacount[GLY]*aamolweights[GLY]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[GLY], aacount[GLY], relcontent, mwcontent);

    relcontent = (float)aacount[ILE]/count;
    mwcontent = ((aacount[ILE]*aamolweights[ILE]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[ILE], aacount[ILE], relcontent, mwcontent);

    relcontent = (float)aacount[LEU]/count;
    mwcontent = ((aacount[LEU]*aamolweights[LEU]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[LEU], aacount[LEU], relcontent, mwcontent);

    relcontent = (float)aacount[MET]/count;
    mwcontent = ((aacount[MET]*aamolweights[MET]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[MET], aacount[MET], relcontent, mwcontent);

    relcontent = (float)aacount[PHE]/count;
    mwcontent = ((aacount[PHE]*aamolweights[PHE]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[PHE], aacount[PHE], relcontent, mwcontent);

    relcontent = (float)aacount[PRO]/count;
    mwcontent = ((aacount[PRO]*aamolweights[PRO]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[PRO], aacount[PRO], relcontent, mwcontent);

    relcontent = (float)aacount[TRP]/count;
    mwcontent = ((aacount[TRP]*aamolweights[TRP]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[TRP], aacount[TRP], relcontent, mwcontent);

    relcontent = (float)aacount[VAL]/count;
    mwcontent = ((aacount[VAL]*aamolweights[VAL]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[VAL], aacount[VAL], relcontent, mwcontent);
  }

  if (printother == TRUE) {
    /* Other ASX GLX UNK*/
    printf("\n# Other side chains\n# AA\tnumber\trelcontent\tMW-content\n");

    if (printstat == TRUE)
    {
      relcontent = (float)othercount/count;
      mwcontent = (othermolweight + (18*relcontent))/molweight;
      printf("all\t%d\t%f\t%f\n", othercount, relcontent, mwcontent);
    }

    relcontent = (float)aacount[ASX]/count;
    mwcontent = ((aacount[ASX]*aamolweights[ASX]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[ASX], aacount[ASX], relcontent, mwcontent);

    relcontent = (float)aacount[GLX]/count;
    mwcontent = ((aacount[GLX]*aamolweights[GLX]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[GLX], aacount[GLX], relcontent, mwcontent);
  }
  
  if (printaroma == TRUE)
  {
    printf("\n# Aromatic side chains\n# AA\tnumber\trelcontent\tMW-content\n");

    if (printstat == TRUE)
    {
      relcontent = (float)aromacount/count;
      mwcontent = (aromamolweight + (18*relcontent))/molweight;
      printf("all\t%d\t%f\t%f\n", aromacount, relcontent, mwcontent);
    }

    relcontent = (float)aacount[PHE]/count;
    mwcontent = ((aacount[PHE]*aamolweights[PHE]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[PHE], aacount[PHE], relcontent, mwcontent);

    relcontent = (float)aacount[TRP]/count;
    mwcontent = ((aacount[TRP]*aamolweights[TRP]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[TRP], aacount[TRP], relcontent, mwcontent);

    relcontent = (float)aacount[TYR]/count;
    mwcontent = ((aacount[TYR]*aamolweights[TYR]) + (18*relcontent))/molweight;
    printf("%s\t%d\t%f\t%f\n", aanames[TYR], aacount[TYR], relcontent, mwcontent);
  }
}
